% This MATLAB script will fit the lognormal perfusion model to a disruption
% replenishment time-intensity curve. 
%
% The time-intensity data should be in an excel document with time in
% column A and linear backscatter data in Column B.
%
% Some sample data entitled 'TIC_noflash.xls' is provided as a template.
%
% Please reference as:
% Hudson et al. Quantification of flow using ultrasound and microbubbles: 
% a disruption replenishment model based on physical principles. 
% Ultrasound in Medicine & Biology (2009) vol. 35 (12) pp. 2007-20
% 
% or
% 
% Hudson et al. The Lognormal Perfusion Model for Disruption Replenishment 
% Measurements of Blood Flow: In Vivo Validation. 
% Ultrasound in Medicine & Biology (2011) vol. 37 (10) pp. 1571-1578
%
% JM Hudson
% March 2013
% *****************************************************************
%

clc,clear,close all

% Read in data from the excel file:
% first column = time, 
% second column = linearized intensity data

[TIC] = xlsread('TIC_noflash.xls');   % <------ Call your data here
time = TIC(:,1); 

% Some pre-processing (remove baseline offset etc)

backscatter_intensity = TIC(:,2)-TIC(1,2);    % subtract baseline
for i= 1:length(backscatter_intensity)
    if backscatter_intensity(i) <0
        backscatter_intensity(i)=0;
    else
    end
end     % remove negative TIC values    

% Call the fitting function and calculate perfusion parameters

HW = 5; % width of the elevation beam in [mm] -> This should be measured separately. Here it is assumed to be 5 mm.
guess = [1 1 0.75 1]; % Initial conditions to seed the fitting function
[A,MV,MU,SIG,delay] = logperf_fit(backscatter_intensity,time',HW,guess,1,1);   % <---- Main fitting function
[FITTED_TIC] = logperf_model(A,MU,SIG,HW,delay,time',1);                        % <---- Main model

% *********** Output perfusion parameters:

A
MV
SIG
MU

% Plot the original data with the fitted curve

figure(1), hold on
plot(time,backscatter_intensity,'o','MarkerSize',4)
plot(time,FITTED_TIC,'-k','LineWidth',2)
ylabel('Backscatter Intensity','FontSize',14),xlabel('Time [s]','FontSize',14), Legend('TIC Data','Lognormal Model')
hold off